#include "MullXCTest/SwiftSupport/SyntaxMutationFinder.h"
#include "MullXCTest/PthreadTaskExecutor.h"
#include <mull/Diagnostics/Diagnostics.h>
#include <mull/Parallelization/Progress.h>
#include <mull/Parallelization/TaskExecutor.h>

using namespace mull_xctest::swift;

class IndexSwiftSourceTask {
public:
  using In = std::set<SourceFilePath>;
  using Out = std::vector<
      std::pair<SourceFilePath, std::unique_ptr<SourceUnitStorage>>>;
  using iterator = In::iterator;

  IndexSwiftSourceTask() {}

  void operator()(iterator begin, iterator end, Out &storage,
                  mull::progress_counter &counter);
};

void IndexSwiftSourceTask::operator()(iterator begin, iterator end,
                                      Out &storage,
                                      mull::progress_counter &counter) {
  for (auto it = begin; it != end; it++, counter.increment()) {
    std::string sourcePath = *it;
    std::unique_ptr<SourceUnitStorage> unit =
        SourceUnitStorage::create(sourcePath);
    if (!unit)
      continue;
    storage.emplace_back(sourcePath, std::move(unit));
  }
}

SourceStorage
SyntaxMutationFinder::findMutations(std::set<SourceFilePath> &sources,
                                    mull::Diagnostics &diagnostics,
                                    const mull::Configuration &config) {
  std::vector<std::pair<SourceFilePath, std::unique_ptr<SourceUnitStorage>>>
      mutationsAsVector;
  std::vector<IndexSwiftSourceTask> tasks(config.parallelization.workers);
  constexpr size_t DesiredStackSize = 8 << 20;
  mull_xctest::PthreadTaskExecutor indexer(diagnostics, "Syntax Index",
                                           DesiredStackSize, sources,
                                           mutationsAsVector, tasks);
  indexer.execute();

  SourceStorage storage;
  for (auto &pair : mutationsAsVector) {
    storage.saveMutations(pair.first, std::move(pair.second));
  }
  return std::move(storage);
}
